package sdtext;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingWorker;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.georgetown.gucs.matcher.CompareFingerprint;
import edu.georgetown.gucs.matcher.ScoreFingerprints;
import edu.georgetown.gucs.utility.Global;

/**
 * Provides an interface for receiving action events related to a matcher; used with {@linkplain GUI}.
 * 
 * @author Lindsay Neubauer
 * 
 */
class MatcherActionListener implements ActionListener {
  JTextArea matcherTextArea = new JTextArea();
  JButton firstLink = new JButton();
  JButton secondLink = new JButton();
  JButton dictionaryLink = new JButton();
  JButton outputLink = new JButton();
  JComboBox matcherChooser = new JComboBox();
  boolean outputScore;
  int minimum_score = 0;
  private JButton compareButton = new JButton();

  MatcherActionListener(boolean outputScore) {
    this.outputScore = outputScore;
    GUI.linkSetup(this.firstLink);
    GUI.linkSetup(this.secondLink);
    GUI.linkSetup(this.dictionaryLink);
    GUI.linkSetup(this.outputLink);

    if (this.outputScore) {
      this.compareButton.setText("Compare Fingerprints");
    } else {
      this.compareButton.setText("Compare to Directory");
    }
    this.compareButton.addActionListener(this);
    this.compareButton.setEnabled(false);

    for (MatcherEnum fingerprinter : MatcherEnum.values()) {
      this.matcherChooser.addItem(fingerprinter);
    }
    this.matcherChooser.setSelectedIndex(0);
    this.matcherChooser.setMaximumSize(new Dimension(200, 50));

    this.matcherTextArea.setFont(Global.unifontFont14);
    this.matcherTextArea.setEditable(false);
  }

  @Override
  public void actionPerformed(final ActionEvent actionEvent) {
    SwingWorker<String, String> worker = new SwingWorker<String, String>() {
      @Override
      protected String doInBackground() {
        ((JButton) actionEvent.getSource()).setEnabled(false);
        MatcherActionListener.this.outputLink.setText("Computing Fingerprints...");
        if (MatcherActionListener.this.firstLink.getActionCommand().length() > 0
            && MatcherActionListener.this.secondLink.getActionCommand().length() > 0) {
          if (!MatcherActionListener.this.outputScore) {
            MatcherActionListener.this.matcherTextArea.setText("");
            if (MatcherActionListener.this.dictionaryLink.getActionCommand().length() == 0
                || MatcherActionListener.this.dictionaryLink.getText().equals("Fingerprint contains dictionary")) {

              try {
                final CompareFingerprint compare = new CompareFingerprint(
                    ((MatcherEnum) MatcherActionListener.this.matcherChooser.getSelectedItem()).name,
                    MatcherActionListener.this.firstLink.getActionCommand(),
                    MatcherActionListener.this.secondLink.getActionCommand(), MatcherActionListener.this.minimum_score,
                    null);
                compare.compareFingerprint();
                MatcherActionListener.this.outputLink.setText("Printing Fingerprints...");
                Iterator<Entry<Integer, List<String>>> scores = compare.iterator();
                boolean printedSomething = false;
                while (scores.hasNext()) {
                  Entry<Integer, List<String>> entry = scores.next();
                  if (entry.getKey() >= MatcherActionListener.this.minimum_score) {
                    for (String file : entry.getValue()) {
                      MatcherActionListener.this.matcherTextArea.append(String.format("%s (score = %d)%n", file,
                          entry.getKey()));
                    }
                    MatcherActionListener.this.matcherTextArea.append(String.format("%n"));
                    printedSomething = true;
                  }
                }
                if (!printedSomething) {
                  MatcherActionListener.this.matcherTextArea
                      .setText("There are no files in the directory with a score higher than "
                          + MatcherActionListener.this.minimum_score);
                }
                MatcherActionListener.this.outputLink.setText("Matching Complete");
              } catch (RuntimeException exception) {
                MatcherActionListener.this.outputLink.setText("");
                Global.monitor.printErrorMessage("MatcherActionListener", "Exception: " + exception.getMessage());
                Global.monitor.writeToBuffer("MatcherActionListener",
                    "compareButton.actionPerformed.compareFingerprint_noDict", exception.getMessage());
                Global.monitor.writeToBuffer("MatcherActionListener",
                    "compareButton.actionPerformed.compareFingerprint_noDict",
                    Arrays.toString(exception.getStackTrace()));
                JOptionPane.showConfirmDialog((Component) actionEvent.getSource(),
                    "Exception: " + exception.getMessage(), "Error: Matcher", JOptionPane.CANCEL_OPTION,
                    JOptionPane.ERROR_MESSAGE);
              }
            } else {
              try {
                final CompareFingerprint compare = new CompareFingerprint(
                    ((MatcherEnum) MatcherActionListener.this.matcherChooser.getSelectedItem()).name,
                    MatcherActionListener.this.firstLink.getActionCommand(),
                    MatcherActionListener.this.secondLink.getActionCommand(), MatcherActionListener.this.minimum_score,
                    MatcherActionListener.this.dictionaryLink.getActionCommand());
                compare.compareFingerprint();
                MatcherActionListener.this.outputLink.setText("Printing Fingerprints...");
                Iterator<Entry<Integer, List<String>>> scores = compare.iterator();
                boolean printedSomething = false;
                while (scores.hasNext()) {
                  Entry<Integer, List<String>> entry = scores.next();
                  if (entry.getKey() >= MatcherActionListener.this.minimum_score) {
                    for (String file : entry.getValue()) {
                      MatcherActionListener.this.matcherTextArea.append(String.format("%s (score = %d)%n", file,
                          entry.getKey()));
                    }
                    MatcherActionListener.this.matcherTextArea.append(String.format("%n"));
                    printedSomething = true;
                  }
                }
                if (!printedSomething) {
                  MatcherActionListener.this.matcherTextArea
                      .setText("There are no files in the directory with a score higher than "
                          + MatcherActionListener.this.minimum_score);
                }
                MatcherActionListener.this.outputLink.setText("Matching Complete");
              } catch (RuntimeException exception) {
                MatcherActionListener.this.outputLink.setText("");
                Global.monitor.printErrorMessage("MatcherActionListener", "Exception: " + exception.getMessage());
                Global.monitor.writeToBuffer("MatcherActionListener",
                    "compareButton.actionPerformed.compareFingerprint_withDict", exception.getMessage());
                Global.monitor.writeToBuffer("MatcherActionListener",
                    "compareButton.actionPerformed.compareFingerprint_withDict",
                    Arrays.toString(exception.getStackTrace()));
                JOptionPane.showConfirmDialog((Component) actionEvent.getSource(),
                    "Exception: " + exception.getMessage(), "Error: Matcher", JOptionPane.CANCEL_OPTION,
                    JOptionPane.ERROR_MESSAGE);
              }
            }
          } else {
            try {
              final ScoreFingerprints score = new ScoreFingerprints(
                  ((MatcherEnum) MatcherActionListener.this.matcherChooser.getSelectedItem()).name,
                  MatcherActionListener.this.firstLink.getActionCommand(),
                  MatcherActionListener.this.secondLink.getActionCommand());
              MatcherActionListener.this.matcherTextArea.append(String.format(
                  "Fingerprint: %s%nFingerprint: %s%nMatch: %s%n%n",
                  MatcherActionListener.this.firstLink.getActionCommand(),
                  MatcherActionListener.this.secondLink.getActionCommand(), score.getMatch()));
              MatcherActionListener.this.matcherTextArea.setCaretPosition(MatcherActionListener.this.matcherTextArea
                  .getDocument().getLength());
              MatcherActionListener.this.outputLink.setText("Matching Complete");
            } catch (RuntimeException exception) {
              MatcherActionListener.this.outputLink.setText("");
              Global.monitor.printErrorMessage("MatcherActionListener", "Exception: " + exception.getMessage());
              Global.monitor.writeToBuffer("MatcherActionListener", "compareButton.actionPerformed.scoreFingerprints",
                  exception.getMessage());
              Global.monitor.writeToBuffer("MatcherActionListener", "compareButton.actionPerformed.scoreFingerprints",
                  Arrays.toString(exception.getStackTrace()));
              JOptionPane.showConfirmDialog((Component) actionEvent.getSource(),
                  "Exception: " + exception.getMessage(), "Error: Matcher", JOptionPane.CANCEL_OPTION,
                  JOptionPane.ERROR_MESSAGE);
            }
          }
        } else {
          MatcherActionListener.this.outputLink.setText("");
          JOptionPane.showConfirmDialog((Component) actionEvent.getSource(), "Must load the fingerprint files",
              "Error: Matching Fingerprints", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
        }
        ((JButton) actionEvent.getSource()).setEnabled(true);
        return "";
      }
    };
    worker.execute();
  }

  /**
   * Creates a panel containing all the necessary components for scoring two fingerprints
   * 
   * @param layout
   *          the GroupLayout to add necessary fingerprint scoring components
   */
  protected void createScorePanel(GroupLayout layout) {
    JPanel matcherPanel = GUI.createSubPanel(Arrays.asList(new JComponent[] { this.matcherChooser }),
        "Fingerprint Matcher", true);
    JPanel firstfingerprinterPanel = GUI.createLoadPanel("First Fingerprint", JFileChooser.FILES_ONLY, this.firstLink,
        null, Global.XML_ONLY);
    JPanel secondfingerprinterPanel = GUI.createLoadPanel("Second Fingerprint", JFileChooser.FILES_ONLY,
        this.secondLink, this.compareButton, Global.XML_ONLY);
    JScrollPane scrollPane = new JScrollPane(this.matcherTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPane.setMinimumSize(new Dimension(200, 200));
    scrollPane.setMaximumSize(new Dimension(5000, 200));

    JPanel outputPanel = GUI.createSubPanel(
        Arrays.asList(new JComponent[] { this.outputLink, this.compareButton, scrollPane }), "Score Fingerprint", true);

    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(matcherPanel)
        .addComponent(firstfingerprinterPanel).addComponent(secondfingerprinterPanel).addComponent(outputPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(matcherPanel)
        .addComponent(firstfingerprinterPanel).addComponent(secondfingerprinterPanel).addComponent(outputPanel));
  }

  /**
   * Creates a panel containing all the necessary components for comparing a fingerprint to a directory
   * 
   * @param layout
   *          the GroupLayout to add necessary fingerprint scoring components
   */
  protected void createComparePanel(GroupLayout layout) {
    JPanel matcherPanel = GUI.createSubPanel(Arrays.asList(new JComponent[] { this.matcherChooser }),
        "Fingerprint Matcher", true);
    final JSpinner spinner = new JSpinner(new SpinnerNumberModel(0, 0, 99, 1));
    spinner.setMaximumSize(new Dimension(100, 20));
    spinner.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent changeEvent) {
        MatcherActionListener.this.minimum_score = (Integer) spinner.getValue();
      }
    });

    final JPanel minScorePanel = GUI.createSubPanel(Arrays.asList(new JComponent[] { spinner }), "Minimum Score", true);
    this.matcherChooser.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent actionEvent) {
        if (((MatcherEnum) MatcherActionListener.this.matcherChooser.getSelectedItem()).name
            .equals("CosineSimilarityFingerprintMatcher")) {
          minScorePanel.setVisible(true);
        } else {
          minScorePanel.setVisible(false);
        }
      }
    });

    this.firstLink.setName("matcher_firstLink");
    JPanel firstfingerprinterPanel = GUI.createLoadPanel("Fingerprint", JFileChooser.FILES_ONLY, this.firstLink,
        this.dictionaryLink, Global.XML_ONLY);
    JPanel directoryPanel = GUI.createLoadPanel("Directory", JFileChooser.DIRECTORIES_ONLY, this.secondLink,
        this.compareButton, Global.ALL_FILES);

    JPanel dictionaryPanel = GUI.createLoadPanel("Dictionary", JFileChooser.FILES_ONLY, this.dictionaryLink, null,
        Global.XML_ONLY);

    JScrollPane scrollPane = new JScrollPane(this.matcherTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPane.setMinimumSize(new Dimension(200, 200));

    JPanel outputPanel = GUI.createSubPanel(
        Arrays.asList(new JComponent[] { this.outputLink, this.compareButton, scrollPane }), "Compare to Directory",
        true);

    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(matcherPanel).addComponent(minScorePanel)
        .addComponent(firstfingerprinterPanel).addComponent(directoryPanel).addComponent(dictionaryPanel)
        .addComponent(outputPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(matcherPanel).addComponent(minScorePanel)
        .addComponent(firstfingerprinterPanel).addComponent(directoryPanel).addComponent(dictionaryPanel)
        .addComponent(outputPanel));
  }
}
